%% Steifigkeit berechnen
clear all;
close all;
clc;

syms k c m
A = [0,1; -k/m,-c/m];

A_eig = eig(A); %Eigenwerte von A
%pretty(A_eig)

% Steifigkeitskoeffizienten bestimmen
m = 1;
c = 1;
k = 2;

A_eig_s = double(subs(A_eig));
disp("Steifigkeit der Systemparameter aus 1.8");
stiff = max(abs(real(A_eig_s))) / min(abs(real(A_eig_s)))

% Steifigkeitskoeffizienten bestimmen mit anderen Werten
m = 1;
c = 1001;
k = 1000;

A_eig_s = double(subs(A_eig));
disp("Steifigkeit der greren Systemparameter");
stiff = max(abs(real(A_eig_s))) / min(abs(real(A_eig_s)))

%% Exakte Lsug steifes System

Ta = 0.005; %Maximum which prints nicely is 0.002
range = 0:Ta:20;

s0 = 1;
v0 = 0.5;
x0 = [s0;v0];

A = [0,1; -k/m,-c/m];
B = [0;1/m];

l1 = -c/(2*m) + sqrt((c/2*m)^2-k/m);
l2 = -c/(2*m) - sqrt((c/2*m)^2-k/m);

t = range;

sympref('HeavisideAtOrigin', 1);
u = @(t) heaviside(t);

%Exakte Loesung aus 1.2.1
y = ((l1+c/m)/(l1-l2)*exp(l1*t) +  (l2+c/m)/(l2-l1)*exp(l2*t))*s0 + ...
         (1/(l1-l2)*exp(l1*t) + 1/(l2-l1)*exp(l2*t))*v0 + ...
         (((1/l1)/(m*(l1-l2)))*(-heaviside(t)+exp(l1*t)))+ ...
         (((1/l2)/(m*(l2-l1)))*(-heaviside(t)+exp(l2*t)));

     
%Integrationsalgorithus Euler explizit
algo_euler = zeros(2,length(range));
algo_euler(:,1)=x0; % Startwert
for n=2:1:length(range)
    algo_euler(:,n) = algo_euler(:,n-1)+Ta*(A*algo_euler(:,n-1)+B*u(range(n-1))); 
end

figure
plot(t,y); %Exakte Lsung
hold on
plot(t,algo_euler(1,:),'r'); %Euler Verfahren
%ylim([0 1])
legend('Exakte Lsung','Explizites Euler Verfahren')
grid on

fprintf("Durchschnittlicher Fehler des expliziten Euler-Verfahrens: %i\n",mean(y-algo_euler(1,:)));

%% Vergleich explizites Eulerverfahren und Exakt
A = [0 1; -k/m -c/m];
B = [0;1/m];
C = [1,0;0,1]; % y = x
D = [0];

sys = ss(A,B,C,D,'InputName','f','OutputName',{'s','v'},'StateName',{'s';'v'});

[y,tt,x] = lsim(sys,u(t),t,x0); %CST kontinuierlich

fprintf("Abtastzeit %i\n",Ta);
fprintf("Durchschnittlicher Fehler des expliziten Euler-Verfahrens Auslenkung: %i\n",mean(x(:,1)'-algo_euler(1,:)));
fprintf("Durchschnittlicher Fehler des expliziten Euler-Verfahrens Geschwindigkeit: %i\n",mean(x(:,2)'-algo_euler(2,:)));


figure
subplot(2,1,1) % Auslenkung
plot(tt,x(:,1));
hold on
grid on
plot(t,algo_euler(1,:),'r'); %Euler Verfahren
legend('Exakte Lsung','Explizites Euler Verfahren')
subplot(2,1,2) % Geschwindigkeit
plot(tt,x(:,2));
hold on
grid on
plot(t,algo_euler(2,:),'r'); %Euler Verfahren
legend('Exakte Lsung','Explizites Euler Verfahren')
%% Smaller X Axis limits
subplot(2,1,1) % Auslenkung
xlim([0,0.05])
subplot(2,1,2) % Geschwindigkeit
xlim([0,0.05])
%%
%Integrationsalgorithus Euler implizit
%Ta = 0.2;
range = 0:Ta:20;
t = range;
[y,tt,x] = lsim(sys,u(t),t,x0); %CST kontinuierlich

algo_ieuler = zeros(2,length(range));
algo_ieuler(:,1)=x0; % Startwert
for n=2:1:length(range)
    algo_ieuler(:,n) = (eye(2)-Ta*A)\(algo_ieuler(:,n-1)+Ta*B*u(range(n))); 
end

fprintf("Abtastzeit %i\n",Ta);
fprintf("Durchschnittlicher Fehler des impliziten Euler-Verfahrens Auslenkung: %i\n",mean(x(:,1)'-algo_ieuler(1,:)));
fprintf("Durchschnittlicher Fehler des impliziten Euler-Verfahrens Geschwindigkeit: %i\n",mean(x(:,2)'-algo_ieuler(2,:)));

figure
subplot(2,1,1) % Auslenkung
plot(tt,x(:,1));
hold on
grid on
plot(t,algo_ieuler(1,:),'r'); %Euler Verfahren
%ylim([0 1])
legend('Exakte Lsung','Impliziertes Euler Verfahren')
subplot(2,1,2) % Geschwindigkeit
plot(tt,x(:,2));
hold on
grid on
plot(t,algo_ieuler(2,:),'r'); %Euler Verfahren
legend('Exakte Lsung','Impliziertes Euler Verfahren')