% Parameterfiles fuer das Simulink Modell des elektrischen Systems 
% (Aufgabe 1.4 und 1.5)
close all;
clear all;
clc;
% Aufgabe 1.4.3
% -------------
% Errechnen Sie die Ruhelage des Systems 1.18!

syms UC1 UC2 UeR UsR R1 R2 C2 K C1

A11 = (-1)/(R1*C1)-1/(R2*C1);
A12 = (1-K)/(R2*C1)-K/(R1*C1);
A21 = 1/(R2*C2);
A22 = (K-2)/(R2*C2);

A  = [A11,A12;A21,A22];
bu = [1/(R1*C1);0];
bd = [0;1/(R2*C2)];
B = [bu, bd];
c  = [0,K];

du = [0;0];
dd = [0;0];
d = [du, dd];

x=[UC1; UC2];
xdot = A*x+bu.*UeR+bd.*UsR;

[UC1r, UC2r] = solve(xdot == [0;0], UC1, UC2);

disp('Ruhelagen des Systems (1.18):');
disp('UC1r =');
pretty(simplify(UC1r));
disp('UC2r =');
pretty(simplify(UC2r));
%%
% Parameter
R1      = 625;    % Widerstand in Ohm
R2      = 3500;   % Widerstand in Ohm
K       = 3;      % Spannungsverstrkung
C1_ref  = 1e-6;   % Referenz-Kapazittswert auf Kennlinie von C1 in F
UC1_ref = -10;    % Referenz-Spannungswert auf Kennlinie von C1 in V
kC1     = 800e-9; % Steigung der Kennlinie von C1 in F/V == C1 abgeleitet!
C2      = 1e-6;   % Kapazitt in F

% Festlegung der Ruhelage in Volt
UeR = 5;
UsR = 5;

%% Eingangsspannung und Stoerspannung (Aufgabe 1.5)
% Sprungfoermige Eingangsspannung
te = 0.5;      % Einschaltzeitpunkt in Sekunden
Ue = 7;      % Endwert des Sprunges in Volt

% Sinusfoermige Eingangsspannung
Usinus = 1; % Amplitude des Sinus in Volt
we     = 10^4;  % Winkelfrequenz des Sinus in rad/s

% Sprungfoermige Stoerspannung
ts = 3;     % Einschaltzeitpunkt in Sekunden
Us = 1;     % Endwert des Sprunges in Volt

%% Modell des linearisierten Systems

% Aufgabe 1.4.4
% -------------
% Errechnen Sie die Ruhelage des Systems!
UC1R = double(subs(UC1r))
UC2R = double(subs(UC2r))
UaR  = double(subs(c*[UC1R;UC2R]))

% Errechnen Sie fr die gegebene Ruhelage die Ableitungen der in C1 
% gespeicherter Ladung Q1 nach der anliegenden Spannung!
Q1 = @(UC1) (C1_ref+kC1*(UC1/2 - UC1_ref))*UC1;
dQ1 = diff(Q1, UC1); % Erste Ableitung von Q1 nach UC1
ddQ1 = diff(dQ1, UC1); % == kC1 % Zweite Ableitung von Q1 nach UC1

Q1   = double(subs(Q1(UC1R)))
dQ1  = double(subs(dQ1,UC1,UC1R))
ddQ1 = double(subs(ddQ1,UC1,UC1R))

% Ergaenzen Sie die Systemmatrix A (Asys), die Eingangsvektoren bu (busys)
% und bd (bdsys) sowie den Ausgangsvektor c (csys) und den Durchgriff d (dsys)
% fr das linearisierte System!

A11 = ((-R1-R2)*(dQ1-ddQ1*UC1R)-ddQ1*((R1*(1-K)-R2*K)*UC2R+R2*UeR))/ ...
        (R1*R2*dQ1^2);
A12 = (-K/(R1*dQ1))+(1-K)/(R2*dQ1);
A21 = 1/(R2*C2);
A22 = (K-2)/(R2*C2);

A  = [A11,A12;A21,A22]
bu = [1/(R1*dQ1);0]
bd = [0;1/(R2*C2)]
c  = [0 K];

du = [0];
dd = [0];

%% Uebertragungsfunktion G (Eingang u -> Ausgang y) und Uebertragungsfunktion Gd (Stoerung d -> Ausgang y)

% Aufgabe 1.4.4
% -------------
% Bestimmen Sie zunchst eine MISO- oder zwei SISO-Zustandsraum- 
% darstellungen mittels ss(). Anschlieend koennen Sie in beiden Faellen
% die gesuchten bertragungsfunktionen (G und Gd) mittels tf() bestimmen.
% MISO ... Multiple Input Single Output
% SISO ... Single Input Single Output

B = [bu,bd];
d = [du,dd];

sys = ss(A,B,c,d,'InputName',{'Ue';'Ud'},'OutputName','Ua', ...
           'InputUnit',{'V','V'},'OutputUnit','V','StateName',{'UC1';'UC2'})

Gall = tf(sys);
G = Gall(1)
Gd = Gall(2)

%% Verstaerkungsfaktor V, Daempfungsgrad xi und Zeitkonstante T von G

% Aufgabe 1.4.5
% -------------
% Bestimmen Sie den Verstaerkungsfaktor V, den Daempfungsgrad xi und die
% Zeitkonstante T der bertragungsfunktion G. 

V  = 1.3766;
T  = 0.001;
xi = 0.8016;

%G2 = tf(V,[T^2 2*xi*T 1]);
%figure
%bode(G2)

%% Bodediagramme der Uebertragungsfunktionen G und Gd

% Aufgabe 1.4.6
% -------------
% Zeichnen und interpretieren Sie die Bodediagramme der beiden
% Uebertragungsfunktionen G und Gd. Verwenden Sie dazu den Befehl bode().
bode(sys)
hold on
grid on