%%
clear all
close all
clc;
Ta = 1.5;
range = 0:Ta:20;

m = 1;
c = 1;
k = 2;
s0 = 1;
v0 = 0.5;
x0 = [s0;v0];

A = [0,1; -k/m,-c/m];
B = [0 1/m];

l1 = -c/(2*m) + sqrt((c/2*m)^2-k/m);
l2 = -c/(2*m) - sqrt((c/2*m)^2-k/m);

t = range;

sympref('HeavisideAtOrigin', 1);
u = @(t) heaviside(t);

%Exakte Loesung aus 1.2.1
y = ((l1+c/m)/(l1-l2)*exp(l1*t) +  (l2+c/m)/(l2-l1)*exp(l2*t))*s0 + ...
         (1/(l1-l2)*exp(l1*t) + 1/(l2-l1)*exp(l2*t))*v0 + ...
         (((1/l1)/(m*(l1-l2)))*(-heaviside(t)+exp(l1*t)))+ ...
         (((1/l2)/(m*(l2-l1)))*(-heaviside(t)+exp(l2*t)));

%Control Systems Toolbox
A = [0 1; -k/m -c/m];
B = [0;1/m];
C = [1,0];
D = 0;

%sys = ss(A,B,C,D)
     
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Integrationsalgorithmen 
%Euler:
algo_euler = zeros(2,length(range));
algo_euler(:,1)=x0; % Startwert
for n=2:1:length(range)
    algo_euler(:,n) = algo_euler(:,n-1)+Ta*(A*algo_euler(:,n-1)+B*u(range(n-1))); 
end

%Heun:
algo_heun = zeros(2,length(range));
algo_heun(:,1)=x0; % Startwert
%Prdikationsschritt = alg_euler
for n=2:1:length(range)
    algo_heun(:,n) = 1/2*algo_heun(:,n-1)+1/2*(algo_euler(:,n)+Ta*(A*algo_euler(:,n)+B*u(range(n)))); 
end

%Runge-Kutta-Verfahren:
algo_runge = zeros(2,length(range));
algo_runge(:,1)=x0; % Startwert
for n=2:1:length(range)
    k1 = (A*algo_runge(:,n-1)+B*u(range(n-1)));
    k2 = (A*(algo_runge(:,n-1)+1/2*Ta*k1)+B*u(range(n)-1/2*Ta));
    k3 = (A*(algo_runge(:,n-1)+1/2*Ta*k2)+B*u(range(n)-1/2*Ta));
    k4 = (A*(algo_runge(:,n-1)+Ta*k3)+B*u(range(n-1)));
    algo_runge(:,n) = algo_runge(:,n-1)+1/6*Ta*(k1+2*k2+2*k3+k4);
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
     
figure
subplot(2,2,1)
plot(t,y); %Exakte Lsung
legend('Exakte Lsung')
grid on
subplot(2,2,2)
plot(t,algo_euler(1,:),'r'); %Euler Verfahren
hold on
plot(t,y); %Exakte Lsung
legend('Euler Verfahren','Exakte Lsung')
grid on
subplot(2,2,3)
plot(t,algo_heun(1,:),'r'); %Heun Verfahren
hold on
plot(t,y); %Exakte Lsung
legend('Heun Verfahren','Exakte Lsung')
grid on
subplot(2,2,4)
plot(t,algo_runge(1,:),'r'); %Runge-Kutta-Verfahren
hold on
plot(t,y); %Exakte Lsung
grid on
legend('Runge-Kutta-Verfahren','Exakte Lsung')

fprintf("Durchschnittlicher Fehler des Euler-Verfahrens: %i\n",mean(y-algo_euler(1,:)));
fprintf("Durchschnittlicher Fehler des Heun-Verfahrens: %i\n",mean(y-algo_heun(1,:)));
fprintf("Durchschnittlicher Fehler des Runge-Kutta-Verfahrens: %i\n",mean(y-algo_runge(1,:)));

