%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Aufgabe 2.1
% FKL Reglerentwurf
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
close all
clear all
clc
s = tf('s');
sympref('HeavisideAtOrigin', 1);

% Strecke:
V = 1.377;
xi = 0.802;
T = 1.002e-3;
%G(s) = V/(1+2*xi*(s*T)+(s*T)^2)
Gs = tf(V,[T^2, 2*xi*T, 1]);

%% 2.1.1 PI Regler:

%A) Spezifikationen:
% Anforderungen an den offenen und geschlossenen Kreis
tr=3e-3;   %Anstiegszeit
ue=5;      %Maximales berschwingen
e_inf = 0; %Bleibende Regelabweichung Sprung

%B) Kenngren:
wc = 1.5/tr; %5.2 Durchtrittsfrequenz
phi = 70-ue; %5.3 prozentuelles berschwingen
roh = 1; %5.9 Anzahl Polstellen bei s=0 um e_inf zu erfllen

Iwc = 1i*wc;

% PI Regler allgemein:
%    R(s) = V_I*(1+sT_I)/s
% 1) Alle bekannten Terme des offenen Kreises == Nur Strecke und Nullstelle bei s=0!
L1 = Gs* 1/s; 
bode(L1);
grid on;
%%
% 2) Bentigte Phasennderung von L1
phase = rad2deg(angle(evalfr(L1,(Iwc)))); % ==> -180-phi
phasen_aenderung = -180+phi-phase;
% --> Phasennderung der brigen Terme (<==> nicht in L1) im PI Regler 
% mssen gleich phasen_aenderung bei wc sein.
% angle(1+ i*wc*TI) == phasen_aenderung
TI = tand(phasen_aenderung)/wc;

L2 = L1 * tf([TI 1],1);
bode(L1, L2);
grid on;
legend;
%Check phase von L2(i*wc) == -180+phi
% Abweichung durch Ungenauigkeit mglich!
%disp("Phasenabweichung korrigiert?");
%abs(rad2deg(angle(evalfr(L2,(Iwc)))) - (-180+phi)) < 1e-10

% 3) Verstrkung VI bestimmen
% Amplitude bei wc == 0dB
VI = 1/abs(evalfr(L2,(Iwc)));

% PI Regler:
%    R(s) = VI*(1+s*TI)/s
disp("Entworfener PI Regler:");
R_PI = VI*tf([TI, 1],[1, 0])

L_PI = Gs * R_PI; %== L2 * VI
bode(L1,L2,L_PI);
grid on;
legend;
%disp("Verstrkung korrigiert?");
%abs(abs(evalfr(L_PI,(Iwc))) - 1) < 1e-10


%% Aufgabe 2.1.2 PI Regler
bode(L_PI,'r');
grid on;

%% Aufgabe 2.1.3 PID Regler

wc = 2*wc;
Iwc = 1i*wc;
% Bleibende Regelabweichung Rampenfunktion
e_inf = 1e-3;

%phi = phi+2.5; %prozentuelles berschwingen erfllen mit PID

% Laut 5.10 ergibt sich e_inf=1/V bei roh=1
% V ... Verstrkung des OFFENEN Kreises L!
VL = 1/e_inf;
% VL/V = VP des PID Reglers und roh=1 bei PID! Laut 5.21
%  Wobei V die Verstrkung der Strecke ist!
VP = VL/V;

% Zeitkonstante des Intagralterms
TI = 1.1e-3;

% PID Regler allgemein:
%    VP * ((1+TI*s)*(1+TD*s))/(s*(1+TR*s))

L1 = Gs * VP*tf([TI, 1],1)*1/s;

% Bentigter Betrag und Phase der brigen Terme bestimmen:
global fun phasen_aenderung betrag_aenderung;
phase = rad2deg(angle(evalfr(L1,(Iwc)))); % ==> -180-phi
phasen_aenderung = -180+phi-phase;
betrag = abs(evalfr(L1,(Iwc)));
betrag_aenderung = 1/betrag;

%Rest = (1+TD*s)/(1+TR*s)
% angle(Rest) == phasen_aenderung
% abs(Rest)   == betrag_aenderung
fun = @(T) (1+T(1)*Iwc)/(1+T(2)*Iwc); %Restterme des Reglers
%Ungleichungen fr fun lsen (Betrag == betrag_aenderung und phase = phase_aenderung)
T = fsolve(@calcTDTR,[0;0]);

%rad2deg(angle(fun(T)))-phasen_aenderung
%abs(fun(T))-betrag_aenderung

TD=T(1);
TR=T(2);

%Gesamtregler und offener Kreis
disp("Entworfener PID Regler:");
R_PID = VP * ((1+TI*s)*(1+TD*s))/(s*(1+TR*s))
L_PID = Gs * R_PID;
bode(L_PID,'g');

%% Aufgabe 2.1.4
% System aus Uebung1 laden
a1_4_3_Esys_Parameter;

% Kontrolle des Verhaltens des geschlossenen Kreise
% siehe Skriptum 4.12
Try_PI = L_PI/(1+L_PI)    % Fhrungsbertragungsfunktion (geschlossener Kreis)
Tdy_PI = Gd/(1+L_PI)     % Strungsbertragungsfunktion (geschlossener Kreis)
Try_PID = L_PID/(1+L_PID)    % Fhrungsbertragungsfunktion (geschlossener Kreis)
Tdy_PID = Gd/(1+L_PID)     % Strungsbertragungsfunktion (geschlossener Kreis)

T = 0.05;   % Simulationsdauer
t=0:0.00001:T;
figure;
subplot(4,1,1);hold on;grid on;
title('Sprungantwort Fhrungsbertragungsfunktion')
plot(t,heaviside(t));
[y,t]=lsim(Try_PI,heaviside(t),t);
plot(t,y);
hold on;
[y,t]=lsim(Try_PID,heaviside(t),t);
plot(t,y);
legend("r","PI","PID");

subplot(4,1,2);hold on;grid on;
title('Rampenantwort Fhrungsbertragungsfunktion')
plot(t,heaviside(t).*t);
[y,t]=lsim(Try_PI,heaviside(t).*t,t);
plot(t,y);
hold on;
[y,t]=lsim(Try_PID,heaviside(t).*t,t);
plot(t,y);
legend("r","PI","PID");

subplot(4,1,3);hold on;grid on;
title('Sprungantwort Strbertragungsfunktion')
plot(t,heaviside(t));
[y,t]=lsim(Tdy_PI,heaviside(t),t);
plot(t,y);
hold on;
[y,t]=lsim(Tdy_PID,heaviside(t),t);
plot(t,y);
legend("d","PI","PID");

subplot(4,1,4);hold on;grid on;
title('Rampenantwort Strbertragungsfunktion')
plot(t,heaviside(t).*t);
[y,t]=lsim(Tdy_PI,heaviside(t).*t,t);
plot(t,y);
hold on;
[y,t]=lsim(Tdy_PID,heaviside(t).*t,t);
plot(t,y);
legend("d","PI","PID");

%% 1.4.5 Stabilitt
isstable(Try_PI)
isstable(Tdy_PI)
isstable(Try_PID)
isstable(Tdy_PID)

%%
figure;
pzmap(Try_PI,Tdy_PI);
grid on;
legend;

figure;
pzmap(Try_PID,Tdy_PID);
grid on;
legend;

%%
% Beschrnkung der Rampe um nichtlineares System zu stabilisieren
% ca 5.5 fr PID Regler
% ca 5.4 fr PI Regler
Ramp_threshold = 5.5;




%% THEORIEFRAGEN!!
% 2.1.2
%% Ist das Frequenzkennlinienverfahren ein exaktes Entwurfsverfahren, wenn -
%% wie im vorliegenden Fall  die Strecke ein Verzgerungsglied 2-ter Ordnung ist?
%   Nein, da die empirisschen Nherungsbeziehungen (Durchtrittsfrequenz, 
%   Phasenreserve) nur fr PT2 als Fhrungsbertragungsfunktion auch exakt
%   berechnet werden knnen. Skriptum Seite 123 

% 2.1.5
%% Ist der geschlossene Regelkreis stabil? 
%   Da der Regler einen Pol bei 0 hat, ist das Gesamtsystem nicht BIBO
%   stabil, da es das Kriterium Re{pol} < 0 fr alle Pole nicht erfllt.
%   Die bertragungsfunktion der Strung, ist stabil, da sie die Polstelle
%   aus der Fhrungsbertragungsfunktion als Nullstelle hat.
%   vgl. Pol- Nullstellen Plot
%% Werden die Anforderungen erfllt?
%   Fr den PI Regler ja, fr den PID regler wird das berschwingen nicht
%   erfllt, da es ca. 7,6% sind. wenn man  um 2,5 erhht, wird die
%   Anforderung erfllt.
stepinfo(Try_PI)
stepinfo(Try_PID)

% 2.1.6
%% Messrauschen
% Das Messrauschen ist am Ausgang sichtbar.
% Durch das Schwanken der Eingangsspannungen, kann es passieren, dass das
% nichtlineare System sich zu weit von der Ruhelage entfernt und gegen
% minus unendlich geht.

% 2.1.7
%% Steigungsbeschrnkung
% Da der Regler mehr Zeit zu Reagieren hat (== die nderungsrate ist begrenzt)
% ist die Stellgre auf den Maxima niedriger.
% Dafr dauert es lnger, bis der Regler den Endwert am Ausgang erreichen kann.
%% Welcher Vorteil kann sich aus der Begrenzung der nderungsrate ergeben?
% Dies kann die Wahrscheinlichkeit einer instabilitt am nichtlinearen
% System verringern, da die Stellgre kleiner ist und das System sich
% damit langsamer von der Ruhelage entfernen kann.

%% Funktion um TD und TR des PID Reglers zu berechnen
function F = calcTDTR(T)
global fun phasen_aenderung betrag_aenderung;

%Gleichungen fr Betrags und Phasenkorrektur, welche == 0 sein sollen:
equ1 = rad2deg(angle(fun(T)))-phasen_aenderung;
equ2 = abs(fun(T))-betrag_aenderung;

F = [equ1; equ2];
end
