close all
clear all
clc
%%
syms phi_GSMP w_GSM w_P L_GSM R_GSM k_GSM J_GSM d_cGSM d_vGSM d_GSMP c_GSMP J_P d_cP d_vP d_qP M_ext u_GSM

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Systemgleichungen x_red'=f(x_red,u,d) == dx
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

x_red=[phi_GSMP; w_GSM; w_P];

i_GSM=1/R_GSM*(u_GSM-k_GSM*w_GSM); %2.10

w_GSMP=w_GSM-w_P;
dw_GSM=1/J_GSM*(k_GSM*i_GSM-d_cGSM-d_vGSM*w_GSM+(w_P-w_GSM)*d_GSMP-phi_GSMP*c_GSMP);
dw_P=1/J_P*((w_GSM-w_P)*d_GSMP+phi_GSMP*c_GSMP-d_cP-d_vP*w_P-d_qP*w_P^2-M_ext);

dx = [w_GSMP;dw_GSM;dw_P];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Ruhelagen des Systems berechnen
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[phi_GSMPr, w_GSMr, w_Pr] = solve(dx == [0;0;0], phi_GSMP, w_GSM, w_P);

disp("Ruhelagen des Systems:");
disp("phi_GSMPr:");
pretty(phi_GSMPr);
disp("w_GSMr:");
pretty(w_GSMr);
disp("w_Pr:");
pretty(w_Pr);

%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Linearisierung
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

A = jacobian(dx, x_red)
%A = subs(A,i_GSM,i_GSMr(1));
%A = subs(A,phi_GSMP,phi_GSMPr(1));
%A = subs(A,w_GSM,w_GSMr(1));
A = subs(A,w_P,w_Pr(2)) %w_Pr > 0!

u = u_GSM;
d = M_ext;

B_u = jacobian(dx, u)
B_d = jacobian(dx, d)

% 2.3.3:
% Da sich in der Ruhelage der Strom ohnehin nicht ndern darf, bleiben die
% Werte der Ruhelagen gleich.

%% 2.5

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Reglerentwurf
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
L_GSM=1.4e-3;
R_GSM=0.46;
k_GSM=0.1;
J_GSM=12.4e-3;
d_cGSM=0.152;
d_vGSM=1.8e-3;
J_P=32.5e-3;
d_cP=0.169;
d_vP=2.7e-3;
d_qP=1e-4;
c_GSMP=0.6822;
d_GSMP=1e-5;

u_GSM=5.6;
M_ext=0;

% Eigenwerte des reduzierten Systems
lambda = eig(subs(A));
disp("l1:");
double(lambda(1))
disp("l2:");
double(lambda(2))
disp("l3:");
double(lambda(3))

%%

Ta = 0.01; % Abtastzeit

sys = ss(double(subs(A)),double(subs([B_u,B_d])),[0,0,1],0);
Gs = tf(sys);

Gz = c2d(Gs(1),Ta,'zoh');
Gq = d2c(Gz,'tustin');

s = tf('s');
q = tf('s');

%% konjugiert komplexes Polpaar bestimmen
[z,p,k] = zpkdata(Gq); %zeros, poles, gain
%conv!

% Per Hand:
%Koeffizientenvergleich fuer xi und T im komplexen Polpaar
%>> (q-p{1}(1))*(q-p{1}(2))
%ans =
%   s^2 + 1.448 s + 75.57
%Continuous-time transfer function.
%
%T=sqrt(1/75.57);
%    T = 0.1150
%xi = 1.448/75.57/2/0.1150;
%    xi = 0.0833

% Mit Matlab:
pl1 = [1 -p{1}(1)]; %(1+q^1 -p(1)*q^0)
pl2 = [1 -p{1}(2)];
%Zwei Polynome gefalten ergeben das Produkt von 2 Polynomen. (q-p1)(q-p2)
Poly = conv(pl1, pl2);
% Produkt umformen auf Form (qT)^2+2*xi(qT)+1
Poly = Poly/Poly(length(Poly));

T = sqrt(Poly(1));
xi = Poly(2)/2/T;

%% Realisierungspole beliebig whlen
% Mssen so gewhlt sein, dass der Regler stabil ist, wenn er die
% Spezifikationen erfllt.
% -7 mehr berschwingen als -6.8
qr1 = -6.8; %-7
qr2 = qr1;

%% Pol-/Nullstellen, L1 bestimmen
%L1 = zpk(z{1},p{1}(3),k)*1/q * 1/((q-qr1)*(q-qr2)) % Konjugiert komplexes Polpaar entfernen, 
L1 = Gq * (1+2*xi*q*T+(q*T)^2) * 1/q * 1/((q-qr1)*(q-qr2));
% mit Terme des Reglers multiplizieren
% zpk(z{1},p{1}(3),k) == Gq ohne konj. kompl. Polpaar
% Restglieder Regler: (VI*(1+q*TI))

tr = 1; %1
wc = 1.2/tr; %6.118
phi = 70-0;
e_inf = 0; % fr r^k 
% 6.124: roh >= 1 -> Pol bei 0

Iwc = 1i*wc;

winkel = rad2deg(angle(evalfr(L1,(Iwc))));
verschiebung = -180+phi-winkel;
TI = tand(verschiebung)/wc;

%check
%L1 = Gq * 1/q * ((1+2*xi*q*T+(q*T)^2)/(q-qr1)*(q-qr2))
L2 = L1 * (1+q*TI); %Berechneten Term zu offenem Kreis hinzufgen
%abs(rad2deg(angle(evalfr(L2,(Iwc)))) - (-180+phi)) < 1e-10 %check

%bode(L1, L2);
%grid on;
%legend("L1","L2");
%% VI bestimmen
% 3) Verstrkung VI bestimmen
% Amplitude bei wc == 0dB
VI = 1/abs(evalfr(L2,(Iwc)));

disp("Kompensationsregler:");
R_komp = VI* ((1+q*TI)/q) * ((1+2*xi*(q*T)+(q*T)^2)/((q-qr1)*(q-qr2)))
L = R_komp * Gq;
Try = L/(1+L);

%Polstellen des geschlossenen Kreises:
%[z,p,k] = tf2zp(1, L.Numerator{1}+ L.Denominator{1})
%alle Re < 0 --> stabil

% Erfllt auch Satz 6.9
figure;
bode(L2, L1, L);
grid on;
legend("L2","L1","L");

%% Parameterfile fr Simulation
GSM_S_Paramter;